@@ -289,7 +289,10 @@ gst_rtpsend_datasink_chain (GstPad *pad, GstBuffer *buf)
           }
           out_buf = gst_netbuffer_new ();
           GST_BUFFER_DATA (out_buf) = GST_BUFFER_DATA(buf);
+          GST_BUFFER_MALLOCDATA(out_buf) = GST_BUFFER_MALLOCDATA(buf);
           GST_BUFFER_SIZE (out_buf) = GST_BUFFER_SIZE(buf);
+          buf = gst_buffer_make_metadata_writable (buf);
+          GST_BUFFER_MALLOCDATA(buf) = NULL;
           gst_netaddress_set_ip4_address (&out_buf->to,
                   dest_ip, dest_port);
           GST_DEBUG_OBJECT(filter, "1. Outgoing RTP packet going straight out without jrtplib %p %d", 
@@ -297,8 +300,9 @@ gst_rtpsend_datasink_chain (GstPad *pad, GstBuffer *buf)
 
           // push data
           gst_pad_push (filter->rtpsrcpad, GST_BUFFER(out_buf));
-          gst_buffer_unref (buf);
       }
+      gst_buffer_unref (buf);
+
 #endif
 
       return GST_FLOW_OK;
