@@ -126,6 +126,9 @@ static gboolean
 gst_live_adder_sink_event (GstPad * pad, GstEvent * event);
 
 
+static void
+reset_pad_private (GstPad *pad);
+
 /* clipping versions */
 #define MAKE_FUNC(name,type,ttype,min,max)                      \
 static void name (type *out, type *in, gint bytes) {            \
@@ -434,6 +437,11 @@ gst_live_adder_flush_start (GstLiveAdder * adder)
   GST_OBJECT_LOCK (adder);
   /* mark ourselves as flushing */
   adder->srcresult = GST_FLOW_WRONG_STATE;
+
+  /* Empty the queue */
+  g_queue_foreach (adder->buffers, (GFunc) gst_mini_object_unref, NULL);
+  while (g_queue_pop_head (adder->buffers)) {}
+
   /* unlock clock, we just unschedule, the entry will be released by the
    * locking streaming thread. */
   if (adder->clock_id)
@@ -443,17 +451,6 @@ gst_live_adder_flush_start (GstLiveAdder * adder)
   GST_OBJECT_UNLOCK (adder);
 }
 
-static void
-gst_live_adder_flush_stop (GstLiveAdder * adder)
-{
-  GST_DEBUG_OBJECT (adder, "Enabling pop on queue");
-
-  /* Mark as non flushing */
-  GST_OBJECT_LOCK (adder);
-  adder->srcresult = GST_FLOW_OK;
-  GST_OBJECT_UNLOCK (adder);
-}
-
 static gboolean
 gst_live_adder_src_activate_push (GstPad * pad, gboolean active)
 {
@@ -463,8 +460,10 @@ gst_live_adder_src_activate_push (GstPad * pad, gboolean active)
   adder = GST_LIVE_ADDER (gst_pad_get_parent (pad));
 
   if (active) {
-    /* allow data processing */
-    gst_live_adder_flush_stop (adder);
+    /* Mark as non flushing */
+    GST_OBJECT_LOCK (adder);
+    adder->srcresult = GST_FLOW_OK;
+    GST_OBJECT_UNLOCK (adder);
 
     /* start pushing out buffers */
     GST_DEBUG_OBJECT (adder, "Starting task on srcpad");
@@ -537,11 +536,14 @@ gst_live_adder_sink_event (GstPad * pad, GstEvent * event)
       ret = gst_pad_push_event (adder->srcpad, event);
       break;
     case GST_EVENT_FLUSH_STOP:
-      ret = gst_pad_push_event (adder->srcpad, event);
-      ret = gst_live_adder_src_activate_push (adder->srcpad, TRUE);
       GST_OBJECT_LOCK (adder);
       adder->segment_pending = TRUE;
+      adder->next_timestamp = GST_CLOCK_TIME_NONE;
+      reset_pad_private (pad);
+      adder->segment_pending = TRUE;
       GST_OBJECT_UNLOCK (adder);
+      ret = gst_pad_push_event (adder->srcpad, event);
+      ret = gst_live_adder_src_activate_push (adder->srcpad, TRUE);
       break;
     case GST_EVENT_EOS:
     {
@@ -583,6 +585,124 @@ newseg_wrong_format:
   }
 }
 
+static gboolean
+gst_live_adder_query_pos_dur (GstLiveAdder * adder, GstFormat informat,
+    gboolean position, gint64 *outvalue)
+{
+  gint64 max = G_MININT64;
+  gboolean res = TRUE;
+  GstIterator *it;
+  gboolean done = FALSE;
+
+
+  it = gst_element_iterate_sink_pads (GST_ELEMENT_CAST (adder));
+  while (!done) {
+    GstIteratorResult ires;
+    gpointer item;
+    GstFormat format = informat;
+
+    ires = gst_iterator_next (it, &item);
+    switch (ires) {
+      case GST_ITERATOR_DONE:
+        done = TRUE;
+        break;
+      case GST_ITERATOR_OK:
+        {
+          GstPad *pad = GST_PAD_CAST (item);
+          gint64 value;
+          gboolean curres;
+
+          /* ask sink peer for duration */
+          if (position)
+            curres = gst_pad_query_peer_position (pad, &format, &value);
+          else
+            curres = gst_pad_query_peer_duration (pad, &format, &value);
+
+          /* take max from all valid return values */
+          /* Only if the format is the one we requested, otherwise ignore it ?
+           */
+
+          if (curres && format == informat) {
+            res &= curres;
+
+            /* valid unknown length, stop searching */
+            if (value == -1) {
+              max = value;
+              done = TRUE;
+            } else if (value > max) {
+              max = value;
+            }
+          }
+          break;
+        }
+      case GST_ITERATOR_RESYNC:
+        max = -1;
+        res = TRUE;
+        break;
+      default:
+        res = FALSE;
+        done = TRUE;
+        break;
+    }
+  }
+  gst_iterator_free (it);
+
+  if (res)
+    *outvalue = max;
+
+  return res;
+}
+
+/* FIXME:
+ *
+ * When we add a new stream (or remove a stream) the duration might
+ * also become invalid again and we need to post a new DURATION
+ * message to notify this fact to the parent.
+ * For now we take the max of all the upstream elements so the simple
+ * cases work at least somewhat.
+ */
+static gboolean
+gst_live_adder_query_duration (GstLiveAdder * adder, GstQuery * query)
+{
+  GstFormat format;
+  gint64 max;
+  gboolean res;
+
+  /* parse format */
+  gst_query_parse_duration (query, &format, NULL);
+
+  res = gst_live_adder_query_pos_dur (adder, format, FALSE, &max);
+
+  if (res) {
+    /* and store the max */
+    gst_query_set_duration (query, format, max);
+  }
+
+  return res;
+}
+
+static gboolean
+gst_live_adder_query_position (GstLiveAdder * adder, GstQuery * query)
+{
+  GstFormat format;
+  gint64 max;
+  gboolean res;
+
+  /* parse format */
+  gst_query_parse_position (query, &format, NULL);
+
+  res = gst_live_adder_query_pos_dur (adder, format, TRUE, &max);
+
+  if (res) {
+    /* and store the max */
+    gst_query_set_position (query, format, max);
+  }
+
+  return res;
+}
+
+
+
 static gboolean
 gst_live_adder_query (GstPad * pad, GstQuery * query)
 {
@@ -664,6 +784,12 @@ gst_live_adder_query (GstPad * pad, GstQuery * query)
       }
       break;
     }
+    case GST_QUERY_DURATION:
+      res = gst_live_adder_query_duration (adder, query);
+      break;
+    case GST_QUERY_POSITION:
+      res = gst_live_adder_query_position (adder, query);
+      break;
     default:
       res = gst_pad_query_default (pad, query);
       break;
@@ -739,10 +865,6 @@ gst_live_adder_src_event (GstPad * pad, GstEvent * event)
       /* TODO : QoS might be tricky */
       result = FALSE;
       break;
-    case GST_EVENT_SEEK:
-      /* TODO : I'm not certain how to handle seeks yet */
-      result = FALSE;
-      break;
     case GST_EVENT_NAVIGATION:
       /* TODO : navigation is rather pointless. */
       result = FALSE;
@@ -1074,7 +1196,10 @@ gst_live_adder_loop (gpointer data)
 
   /* If we have no clock, then we can't do anything.. error */
   if (!clock) {
-    goto no_clock;
+    if (adder->playing)
+      goto no_clock;
+    else
+      goto push_buffer;
   }
 
   GST_DEBUG_OBJECT (adder, "sync to timestamp %" GST_TIME_FORMAT,
@@ -1113,6 +1238,8 @@ gst_live_adder_loop (gpointer data)
   if (ret != GST_CLOCK_OK && ret != GST_CLOCK_EARLY)
     goto clock_error;
 
+ push_buffer:
+
   buffer = g_queue_pop_head (adder->buffers);
 
   if (!buffer)
@@ -1125,8 +1252,12 @@ gst_live_adder_loop (gpointer data)
    */
   if (GST_CLOCK_TIME_IS_VALID (adder->next_timestamp) &&
       GST_BUFFER_TIMESTAMP (buffer) != adder->next_timestamp) {
-    if (llabs (GST_BUFFER_TIMESTAMP (buffer) - adder->next_timestamp) <
-        GST_SECOND / adder->rate) {
+    GstClockTimeDiff diff = GST_CLOCK_DIFF (GST_BUFFER_TIMESTAMP (buffer),
+        adder->next_timestamp);
+    if (diff < 0)
+      diff = -diff;
+
+    if (diff < GST_SECOND / adder->rate) {
       GST_BUFFER_TIMESTAMP (buffer) = adder->next_timestamp;
       GST_DEBUG_OBJECT (adder, "Correcting slight skew");
       GST_BUFFER_FLAG_UNSET(buffer, GST_BUFFER_FLAG_DISCONT);
@@ -1327,9 +1458,8 @@ gst_live_adder_release_pad (GstElement * element, GstPad * pad)
 }
 
 static void
-reset_pad_private (gpointer data, gpointer unused)
+reset_pad_private (GstPad *pad)
 {
-  GstPad *pad = data;
   GstLiveAdderPadPrivate *padprivate;
 
   padprivate = gst_pad_get_element_private (pad);
@@ -1353,21 +1483,34 @@ gst_live_adder_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      {
-        GST_OBJECT_LOCK (adder);
-        adder->segment_pending = TRUE;
-        adder->peer_latency = 0;
-        adder->next_timestamp = GST_CLOCK_TIME_NONE;
-        g_list_foreach (adder->sinkpads, reset_pad_private, NULL);
-        GST_OBJECT_UNLOCK (adder);
-        break;
-      }
+      GST_OBJECT_LOCK (adder);
+      adder->segment_pending = TRUE;
+      adder->peer_latency = 0;
+      adder->next_timestamp = GST_CLOCK_TIME_NONE;
+      g_list_foreach (adder->sinkpads, (GFunc) reset_pad_private, NULL);
+      GST_OBJECT_UNLOCK (adder);
+      break;
+    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
+      GST_OBJECT_LOCK (adder);
+      adder->playing = FALSE;
+      GST_OBJECT_UNLOCK (adder);
+      break;
     default:
       break;
   }
 
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
+  switch (transition) {
+    case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
+      GST_OBJECT_LOCK (adder);
+      adder->playing = TRUE;
+      GST_OBJECT_UNLOCK (adder);
+      break;
+    default:
+      break;
+  }
+
   return ret;
 }
 
