@@ -1012,7 +1012,7 @@ gst_rtp_jitter_buffer_chain (GstPad * pad, GstBuffer * buffer)
         GST_DEBUG_OBJECT (jitterbuffer, "dropping buffer down to 0.5 %u %u", latency_ts_units,
             async_jitter_queue_length_ts_units_unlocked (priv->queue));
 
-        old_buf = async_jitter_queue_pop_unlocked (priv->queue);
+        old_buf = async_jitter_queue_pop_unlocked_forced (priv->queue);
         if (old_buf)
           gst_buffer_unref (old_buf);
       }
